#!/usr/bin/perl

#		UpdateCGI NewsPro addon update tracker
#
#		by Darya Misse
#		plushpuffin@yahoo.com
#		http://plushpuffin.virtualave.net
#
#
#
#
#	This script is intended to be used with these two arguments:
#	1) version
#	2) item
#
#	'version' is a number, either decimal or integral.
#	'item' is the name of an item in your NewsPro/Coranto database.
#
#	eg:
#	http://plushpuffin.virtualave.net/update/update.cgi?item=cra_maginot&version=1.0
#
#	Play around with the above URL, changing the version number and
#	refreshing the page. You can see that the output will change.
#
#	How to use:
#	1) set up NewsPro or Coranto.
#	2) create a new field called CustomField_Version.
#	3) submit news.
#	3) edit the settings below according to your server settings.
#
#	4) pay particular attention to the SplitDataFile subroutine.
#		if you're using NewsPro 3, you should replace it with
#		the SplitDataFile subroutine from your npconfig.pl file.
#		Make sure you change "$newssubject" to be "$Subject"
#
#####################################################################

$IIS = 0;
#	MICROSOFT IIS COMPATIBILITY
#	Set the following variable to one of the following:
#	$IIS = 0; # The default - auto-detect. Usually works fine.
#	$IIS = 1; # You're on IIS -- do basic IIS compatibility.
#	$IIS = 2; # You're on IIS, and you still had problems when $IIS was 1.
#	$IIS = -1; # You're not on IIS - disable even the auto-detection.



$update_images = 'http://plushpuffin.virtualave.net/update';
#	Set the above to the URL to UpdateCGI's image directory.
#	This is usually the directory containing UpdateCGI itself.
#	Do not use a trailing slash. Example:
#	$update_images = 'http://my.site.com/update';


$newsdatfile = '/data1/virtualave.net/plushpuffin/public_html/update/newsdat.txt';
#	*** NEWSDAT.TXT LOCATION ***
#	Set the absolute path to your newsdat.txt file.
#	Use an absolute path (no trailing slash).
#	Example:
#	$newsdatfile = '/absolute/path/to/newspro-admin/newsdat.txt';


sub SplitDataFile {
	($Subject, $User, $Text, $newsid, $newstime, $Category, $CustomField_Version) = split(/\`\`x/, $_[0]);
}
#	*** SplitDataFile format ***
#	Above is the format for SplitDataFile.
#	You may edit it if your fields are in a different order.
#	(this is accessible in either npconfig.pl for NP3, or
#	in crcfg.pl for NP4/Coranto)
#
#	Please note that the only two fields UpdateCGI cares about
#	are Subject and CustomField_Version
#
#

#$DebugFeatures = 1;
#	Uncomment if debugging. Otherwise leave this alone.
#	When debugging, you cannot use this script normally:
#	You must type the GET request into your browser's
#	address bar, and it will return a full HTML page
#	with debugging information.
#

#######
# START (Unless you know Perl, don't change anything after this point.)
#######

$update_cgi_build = 1;

# In order to trap as many errors as possible, we run everything via an eval.
# If $JustLoadSubs is set, that means we're being included by an external
# script which doesn't want us to run, so don't.

unless ($JustLoadSubs) {
	# Disable warnings and redirect warn signals to fatal errors.
	$^W = 0;
	$SIG{__WARN__} = sub { die @_; };
	eval { 
		main(); 
	};
	if ($@) {
		UPdie("Untrapped Error: $@");
	}
}

sub main {	
	# Try and be compatible with Microsoft IIS.
	if ($IIS >= 0) {
		if ($IIS == 0) {
			if ($ENV{'SERVER_SOFTWARE'} =~ /IIS/) {
				$IIS = 1; 
			}
		}
		if (($IIS) && ($0 =~ m!(.*)(\\|\/)!)) {
			chdir($1);
		}
		if ($IIS == 2 && $DebugFeatures) {
			print "HTTP/1.0 200 OK\n";
		}
	}
	RunUpdateCGI();
}

sub RunUpdateCGI {
	# Get form input
	ReadForm();

	$update_status = q~unknown.gif~;

	# Debugging feature: show configuration (no passwords are displayed).
	if ($in{item} && $in{version} ) {
		open( NDFH, $newsdatfile);
		NCLOOP: while (<NDFH>) {
			print qq~this : $_~;
			chomp($_);
			# Split the item into the news fields
			SplitDataFile($_);
			if( $Subject eq $in{item} ) {
				$update_status = $in{version} < $CustomField_Version ? q~obsolete.gif~ : q~uptodate.gif~;
				last;
			}
		}
		close(NDFH);
	}
	if( $DebugFeatures ) {
		print qq~Content-type: text/html\n
<html><head><title>$update_status</title></head><body>
<hr>
in(item)    = $in{item}<br>
in(version) = $in{version}<br>
<hr>
subject = $Subject<br>
version = $CustomField_Version<br>
<hr>
result = $update_status<br>
<hr>
</body></html>
~;
		$HeaderPrinted = 1;
		$HTMLHeaderPrinted = 1;
		exit;
	}
	print qq~Location: $update_images/$update_status\n\n~;
}

# ReadForm: Reads in form values to %in
sub ReadForm {
	my (@pairs, $buffer, $pair, $name, $value); 
	if ($ENV{'REQUEST_METHOD'} eq 'GET') {
		@pairs = split(/&/, $ENV{'QUERY_STRING'});
	}
	elsif ($ENV{'REQUEST_METHOD'} eq 'POST') {
		read(STDIN, $buffer, $ENV{'CONTENT_LENGTH'});
		@pairs = split(/&/, $buffer);
	}
	foreach $pair (@pairs) {
		($name, $value) = split(/=/, $pair);
	
		$name =~ tr/+/ /;
		$name =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
	
		$value =~ tr/+/ /;
		$value =~ s/%([a-fA-F0-9][a-fA-F0-9])/pack("C", hex($1))/eg;
	
		exists $in{$name} ? ($in{$name} .= "|x|$value") : ($in{$name}  = $value);
	}
}

# UPdie: Generates a pretty error message with as much information as possible.
sub UPdie {
	if ($UPdied) {
		exit;
	}
	$UPdied++;
	my $msg = shift;
	my $enableHTML = shift;
	$msg =~ s/\n//g;
	$msg =~ s/<br>/\n/g;
	unless ($enableHTML) {
		$msg =~ s/</&lt;/g;
		$msg =~ s/>/&gt;/g;
	}
	$msg =~ s/\n/<br>/g;
	if (!$HeaderPrinted) {
		print "Content-type: text/html\n\n";
	}
	if (!$HTMLHeaderPrinted) {
		print qq~<html><head><title>UpdateCGI: Fatal Error</title>
<meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
</head>
<body link="#D00000" vlink="#D00000" alink="#D00000" class="bodybg">
~;
	}
	print qq~<div align="center">An error has occurred. The exact description of the error is:</div>
	<table width="80%" border="0" cellpadding="2" align="center"><tr class="fieldtitle"><td class="whitebg">
	<div align="center">$msg</div></td></tr></table>
	<table width="90%" border="0" align="center" cellpadding="3" class="yellowbg"><tr><td class="footnote"><hr><div align="center"><b>USEFUL INFORMATION</b></div>\n<br>~;
	if ($!) {
		print "Perl may have generated the following error: $!\n<br>";
	}
	print "Perl Version: $]\n<br>";
	print "Script Version: $UpdateCGIVersion\n<br>";
	print "Script Build: $coreBuild\n<br>";
	print "Script URL: $scripturl\n<br>";
	print '@INC: <ul><li>' . join('</li><li>', @INC) . '</li></ul>';
	($0 =~ m,(.*)/[^/]+,)   && unshift (@INC, "$1");
	($0 =~ m,(.*)\\[^\\]+,) && unshift (@INC, "$1");
	print "Script Location (Method 1): $0\n<br>";
	print "<br>\n<b>ENVIRONMENT VARIABLES</b>\n<br>";
	while (($key, $value) = each %ENV) {
		unless ($key eq "HTTP_COOKIE") {
			print "$key: $value\n<br>";
		}
	}
	print "<br>\n<b>MESSAGE:</b> $msg\n<br>";
	print '<hr></td></tr></table></body></html>';
	exit;
}
1;